@@ -1,11 +1,11 @@
 package Net::DNS::SEC;
 
 #
-# $Id: SEC.pm 1659 2018-03-30 10:32:24Z willem $
+# $Id: SEC.pm 1665 2018-04-05 10:21:44Z willem $
 #
 our $VERSION;
-$VERSION = '1.06_01';
-our $SVNVERSION = (qw$LastChangedRevision: 1659 $)[1];
+$VERSION = '1.06_02';
+our $SVNVERSION = (qw$LastChangedRevision: 1665 $)[1];
 our $XS_VERSION = $VERSION;
 $VERSION = eval($VERSION);
 
@@ -24,6 +24,11 @@ The Net::DNS::SEC suite provides additional packages required to support
 DNSSEC as described in RFC4033, RFC4034, RFC4035 and subsequent related
 documents.
 
+It also provides support for SIG0 which is useful for dynamic updates
+using key-pairs.
+
+RSA, DSA and elliptic curve crypto algorithms are supported.
+
 Net::DNS::SEC is installed as an extension to an existing Net::DNS
 installation.
 
