@@ -1,10 +1,9 @@
 package Net::DNS::SEC::ECDSA;
 
 #
-# $Id: ECDSA.pm 1619 2018-01-24 08:24:17Z willem $
+# $Id: ECDSA.pm 1660 2018-04-03 14:12:42Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1619 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1660 $)[1];
 
 
 =head1 NAME
@@ -30,14 +29,14 @@ generation and verification procedures.
 
     $signature = Net::DNS::SEC::ECDSA->sign( $sigdata, $private );
 
-Generates the wire-format binary signature from the binary sigdata
+Generates the wire-format signature from the sigdata octet string
 and the appropriate private key object.
 
 =head2 verify
 
     $validated = Net::DNS::SEC::ECDSA->verify( $sigdata, $keyrr, $sigbin );
 
-Verifies the signature over the binary sigdata using the specified
+Verifies the signature over the sigdata octet string using the specified
 public key resource record.
 
 =cut
@@ -45,7 +44,6 @@ public key resource record.
 use strict;
 use integer;
 use warnings;
-use Carp;
 use Digest::SHA;
 use MIME::Base64;
 
@@ -55,16 +53,6 @@ my %ECDSA = (
 	);
 
 
-my %ECkey;
-
-sub _curve {
-	my $nid = shift;
-	my $key = $ECkey{$nid};
-	return $key if $key;
-	$ECkey{$nid} = Net::DNS::SEC::libcrypto::EC_KEY_new_by_curve_name($nid);
-}
-
-
 sub sign {
 	my ( $class, $sigdata, $private ) = @_;
 
@@ -75,30 +63,25 @@ sub sign {
 	$hash->add($sigdata);
 	my $digest = $hash->digest;
 
-	my $keybin = decode_base64( $private->PrivateKey );
+	my $key = decode_base64( $private->PrivateKey );	# private key
+	my $len = length $key;
 
 	my $eckey = Net::DNS::SEC::libcrypto::EC_KEY_dup( _curve($NID) );
-	Net::DNS::SEC::libcrypto::EC_KEY_set_private_key( $eckey, $keybin );
+	Net::DNS::SEC::libcrypto::EC_KEY_set_private_key( $eckey, $key );
 
-	my $size = length $keybin;
-	$keybin = $digest;					# destroy private key
-
-	my $sig = Net::DNS::SEC::libcrypto::ECDSA_do_sign( $digest, $eckey );
+	my $sig = Net::DNS::SEC::libcrypto::ECDSA_sign( $digest, $eckey );
 
 	Net::DNS::SEC::libcrypto::EC_KEY_free($eckey);		# destroy private key
 
-	my $z;
-	if ($sig) {						# uncoverable branch false
-		my ( $r, $s ) = Net::DNS::SEC::libcrypto::ECDSA_SIG_get0($sig);
-
-		# both the R and S parameters need to be zero padded:
-		my $Rpad = $size - length($r);
-		my $Spad = $size - length($s);
-		$z = pack "x$Rpad a* x$Spad a*", $r, $s;
-	}
+	return unless $sig;					# uncoverable branch true
 
+	my ( $r, $s ) = Net::DNS::SEC::libcrypto::ECDSA_SIG_get0($sig);
 	Net::DNS::SEC::libcrypto::ECDSA_SIG_free($sig);
-	return $z;
+
+	# both the R and S parameters need to be zero padded:
+	my $Rpad = $len - length($r);
+	my $Spad = $len - length($s);
+	pack "x$Rpad a* x$Spad a*", $r, $s;
 }
 
 
@@ -110,24 +93,23 @@ sub verify {
 	die 'public key not ECDSA' unless $object;
 	my $hash = $object->new(@param);
 	$hash->add($sigdata);
-
-	my $keybin = $keyrr->keybin;				# public key
-	my $keylen = length($keybin) >> 1;
-	my ( $x, $y ) = unpack "a$keylen a*", $keybin;
+	my $digest = $hash->digest;
 
 	return unless $sigbin;
 
+	my $key = $keyrr->keybin;				# public key
+	my $len = length($key) >> 1;
+	my ( $x, $y ) = unpack "a$len a*", $key;
+
 	my $eckey = Net::DNS::SEC::libcrypto::EC_KEY_dup( _curve($NID) );
 	Net::DNS::SEC::libcrypto::EC_KEY_set_public_key_affine_coordinates( $eckey, $x, $y );
 
-	my $siglen = length($sigbin) >> 1;			# signature
-	my ( $r, $s ) = unpack( "a$siglen a*", $sigbin );
+	my ( $r, $s ) = unpack( "a$len a*", $sigbin );		# signature
 
 	my $sig = Net::DNS::SEC::libcrypto::ECDSA_SIG_new();
-
 	Net::DNS::SEC::libcrypto::ECDSA_SIG_set0( $sig, $r, $s );
 
-	my $vrfy = Net::DNS::SEC::libcrypto::ECDSA_do_verify( $hash->digest, $sig, $eckey );
+	my $vrfy = Net::DNS::SEC::libcrypto::ECDSA_verify( $digest, $sig, $eckey );
 
 	Net::DNS::SEC::libcrypto::EC_KEY_free($eckey);
 	Net::DNS::SEC::libcrypto::ECDSA_SIG_free($sig);
@@ -135,6 +117,25 @@ sub verify {
 }
 
 
+########################################
+
+{
+	my %ECkey;
+
+	sub _curve {
+		my $nid = shift;
+		return $ECkey{$nid} if $ECkey{$nid};
+		$ECkey{$nid} = Net::DNS::SEC::libcrypto::EC_KEY_new_by_curve_name($nid);
+	}
+
+	END {
+		foreach ( grep defined, values %ECkey ) {
+			Net::DNS::SEC::libcrypto::EC_KEY_free($_);
+		}
+	}
+}
+
+
 1;
 
 __END__
